@@ -397,7 +397,7 @@ static int get_file_sizes(struct thread_data *td)
 		dprint(FD_FILE, "get file size for %p/%d/%p\n", f, i,
 								f->file_name);
 
-		if (td->io_ops->get_file_size(td, f)) {
+		if (td_io_get_file_size(td, f)) {
 			if (td->error != ENOENT) {
 				log_err("%s\n", td->verror);
 				err = 1;
@@ -686,7 +686,7 @@ int add_file(struct thread_data *td, const char *fname)
 	f->fd = -1;
 
 	if (td->files_size <= td->files_index) {
-		int new_size = td->o.nr_files;
+		int new_size = td->o.nr_files + 1;
 
 		dprint(FD_FILE, "resize file array to %d files\n", new_size);
 
